import sys
from setuptools import setup

if sys.platform != "linux":
    # Refuse to install from the source distribution if it's not linux.
    # When you want to support windows later, just delete this `setup.py` file 
    # or remove this check.
    import warnings
    warnings.warn(
        "\n\n*** ACCELERA INSTALLATION ERROR ***\n"
        "This version of accelera is only supported on Linux. "
        "Windows support will be added in a future release.\n"
        "***********************************\n"
    )
    sys.exit(1)

setup()
